@@ -1,5 +1,5 @@
-# $Date: 2007-07-11 07:47:37 -0700 (Wed, 11 Jul 2007) $
-# $Revision: 148 $
+# $Date: 2007-07-15 03:11:41 -0700 (Sun, 15 Jul 2007) $
+# $Revision: 159 $
 # $Author: david.romano $
 # ex: set ts=8 sw=4 et
 #########################################################################
@@ -9,16 +9,13 @@ use warnings;
 use strict;
 use Carp;
 
-use version; our $VERSION = qv('0.4.1');
+use version; our $VERSION = qv('0.4.2');
 
 # always return an array reference. The value returned by Facebook is a hash
 # reference when the are no results, so this shouldn't be a problem.
 sub query {
     my $self = shift;
-    my $response = $self->base->call( 'fql.query', @_ );
-
-    return $response if !$self->base->parse && $self->base->format ne 'JSON';
-    return ref $response eq 'HASH' ? [] : $response;
+    return $self->base->call( 'fql.query', @_ );
 }
 
 1;    # Magic true value required at end of module
@@ -30,7 +27,7 @@ WWW::Facebook::API::FQL - Facebook Query Language
 
 =head1 VERSION
 
-This document describes WWW::Facebook::API::FQL version 0.4.1
+This document describes WWW::Facebook::API::FQL version 0.4.2
 
 =head1 SYNOPSIS
 
@@ -50,10 +47,7 @@ The fql.query method of the Facebook API:
 
     $response = $client->fql->query( query => 'FQL query' );
 
-When C<< $client->base->parse >> returns true, and when the format is JSON,
-the response will always be an array reference. (Facebook actually returns an
-empty hash reference when empty, but an array reference otherwise. This is
-meant to make developer more consistent.)
+Returns nothing if no result returned from Facebook.
 
 =back
 
